###################################################################################################
# _zreferableitem.py
#
# Implementation of classes ZReferableItem (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTML, HTMLFile
import string
import time
import urllib
# Product Imports.
import _globals
import _objattrs


# ---------------------------------------------------------------------------------------------
# isMailLink:
# ---------------------------------------------------------------------------------------------
def isMailLink(url):
  return string.find(string.lower(url),'mailto:') == 0

# ---------------------------------------------------------------------------------------------
# isInternalLink:
# ---------------------------------------------------------------------------------------------
def isInternalLink(url):
  return url is not None and len(url)>=3 and url[:2]=='{$' and url[-1]=='}'


###################################################################################################
###################################################################################################
###
###   class ZReferableItem
###
###################################################################################################
###################################################################################################
class ZReferableItem: 

  # Management Permissions.
  # -----------------------
  __authorPermissions__ = (
		'manage_RefForm',
		)
  __editorPermissions__ = (
		)
  __administratorPermissions__ = (
		)
  __ac_permissions__=(
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		('ZMS Administrator', __administratorPermissions__),
		)

  # Management Interface.
  # ---------------------
  manage_RefForm = HTMLFile('dtml/zmslinkelement/manage_refform', globals())
  browse_objs = HTMLFile('dtml/zmslinkelement/browse_objs', globals()) 

  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.getRefObjPath:
  # ---------------------------------------------------------------------------------------------
  def getRefObjPath(self, ob):
    path = ob.relative_obj_path()
    homeId = ob.getHome().id
    if homeId != self.getHome().id:
      path = homeId + '@' + path
    ref = '{$' + path + '}'
    return ref


  """
  ###############################################################################################
  ###  
  ###  Links FROM other objects.
  ### 
  ###############################################################################################
  """

  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.getRefByObjs:
  #
  #  Returns references BY other objects.
  # ---------------------------------------------------------------------------------------------
  def getRefByObjs(self, REQUEST):
    ref_by = []
    if _objattrs.hasobjattr(self,'ref_by'):
      ref_by = getattr(self,'ref_by',[])
    return ref_by


  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.synchronizeRefByObjs:
  #
  #  Synchronizes references BY other objects.
  # ---------------------------------------------------------------------------------------------
  def synchronizeRefByObjs(self):
    key = 'ref_by'
    v = getattr(self,key,[])
    ref_by = []
    for i in v:
      ob = self.getLinkObj(i)
      if ob is not None:
        #++ if _globals.debug: print "[%s.synchronizeRefByObjs]: Referenced by %s.%s=%s"%(self.meta_type,self.id,key,i)
        ref_by.append(self.getRefObjPath(ob))
    setattr(self,'ref_by',ref_by)


  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.clearRegisteredRefObjs:
  #
  #  Clears registered referencing objects.
  # ---------------------------------------------------------------------------------------------
  def clearRegisteredRefObjs(self, REQUEST):
    ref_by = []
    setattr(self,'ref_by',ref_by)

    
  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.registerRefObj:
  #
  #  Registers referencing object.
  # ---------------------------------------------------------------------------------------------
  def registerRefObj(self, ob, REQUEST):
    #++ if _globals.debug: print "[%s.registerRefObj]: %s(%s)"%(self.meta_type,ob.id,ob.meta_type)
    ref = self.getRefObjPath(ob)
    ref_by = self.getRefByObjs(REQUEST)
    if not ref in ref_by:
      ref_by.append(ref)
    
    ##### Set Property ####
    setattr(self,'ref_by',ref_by)


  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.unregisterRefObj:
  #
  #  Unregisters referencing object.
  # ---------------------------------------------------------------------------------------------
  def unregisterRefObj(self, ob, REQUEST):
    #++ if _globals.debug: print "[%s.unregisterRefObj]: %s(%s)"%(self.meta_type,ob.id,ob.meta_type)
    prefix = '{$'
    homeId = ob.getHome().id
    if homeId != self.getHome().id:
      prefix = prefix + homeId + '@'
    suffix = ob.id + '}'
    ref_by = self.getRefByObjs(REQUEST)
    for url in ref_by:
      if len(url) >= len(prefix+suffix) and \
         url[:len(prefix)] == prefix and \
         url[-len(suffix):] == suffix:
        del ref_by[ref_by.index(url)]
        
    ##### Set Property ####
    setattr(self,'ref_by',ref_by)


  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.refreshRefObj:
  #
  #  Refreshs referencing object.
  # ---------------------------------------------------------------------------------------------
  def refreshRefObj(self, ob, REQUEST):
    self.unregisterRefObj(ob,REQUEST)
    self.registerRefObj(ob,REQUEST)

    
  """
  ###############################################################################################
  ###  
  ###  References TO other objects.
  ### 
  ###############################################################################################
  """

  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.getRefToObjs:
  #
  #  Returns list of references TO other objects.
  # ---------------------------------------------------------------------------------------------
  def getRefToObjs(self, REQUEST):
    ref_to =  []
    for key in self.getObjAttrs().keys():
      obj_attr = self.getObjAttr(key)
      datatype = self.getObjDatatypeKey(obj_attr)
      if datatype == _globals.DT_URL:
        ref = self.getObjProperty(key,REQUEST)
        if not ref in ref_to:
          ref_to.append(ref)
    return ref_to


  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.synchronizeRefByObjs:
  #
  #  Synchronizes references TO other objects.
  # ---------------------------------------------------------------------------------------------
  def synchronizeRefToObjs(self):
    #++ if _globals.debug: print "[%s.synchronizeRefToObjs]:"%(self.meta_type)
    for key in self.getObjAttrs().keys():
      obj_attr = self.getObjAttr(key)
      datatype = self.getObjDatatypeKey(obj_attr)
      if datatype == _globals.DT_URL:
        for s_lang in self.getLanguages():
          req = {'lang':s_lang,'preview':'preview'}
          ref = self.getObjProperty(key,req)
          ref_obj = self.getLinkObj(ref)
          if ref_obj is not None:
            #++ if _globals.debug: print "[%s.synchronizeRefToObjs]: Reference to %s.%s_%s=%s(%s)"%(self.meta_type,self.id,key,s_lang,ref_obj.id,ref_obj.meta_type)
            ref_obj.refreshRefObj(self,req)


  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.unregisterRefToObj:
  #
  #  Unregisters reference TO given destination.
  # ---------------------------------------------------------------------------------------------
  def unregisterRefToObj(self, dest_obj, REQUEST):
    #++ if _globals.debug: print "[%s.unregisterRefToObj]: %s(%s)"%(self.meta_type,dest_obj.id,dest_obj.meta_type)
    ref_to =  []
    for key in self.getObjAttrs().keys():
      obj_attr = self.getObjAttr(key)
      datatype = self.getObjDatatypeKey(obj_attr)
      if datatype == _globals.DT_URL:
        ref = self.getObjProperty(key,REQUEST)
        ref_obj = self.getLinkObj(ref)
        if ref_obj is not None:
          if self.getRefObjPath(dest_obj) == self.getRefObjPath(ref_obj):
            ##### Set Property ####
            self.setObjStateModified(REQUEST)
            self.setObjProperty(key,None,REQUEST['lang'])


  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.refreshRefToObj:
  #
  #  Refreshs reference TO given destination.
  # ---------------------------------------------------------------------------------------------
  def refreshRefToObj(self, dest_obj, REQUEST):
    #++ if _globals.debug: print "[%s.refreshRefToObj]: %s(%s)"%(self.meta_type,dest_obj.id,dest_obj.meta_type)
    ref_to =  []
    for key in self.getObjAttrs().keys():
      obj_attr = self.getObjAttr(key)
      datatype = self.getObjDatatypeKey(obj_attr)
      if datatype == _globals.DT_URL:
        ref = self.getObjProperty(key,REQUEST)
        ref_obj = self.getLinkObj(ref)
        if ref_obj is not None:
          if self.getRefObjPath(dest_obj) == self.getRefObjPath(ref_obj):
            ##### Set Property ####
            self.setObjStateModified(REQUEST)
            self.setObjProperty(key,self.getRefObjPath(dest_obj),REQUEST['lang'])


  """
  ###############################################################################################
  ###  
  ###  Process Events 
  ### 
  ###############################################################################################
  """

  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.onDeleteRefObjs:
  #
  #  This method is executed before an object is deleted.
  # ---------------------------------------------------------------------------------------------
  def onDeleteRefObjs(self, REQUEST, deep=0):
    #++ if _globals.debug: print "[%s.onDeleteRefObjs]: %s"%(self.meta_type,self.id)
    
    ##### Unregister references TO other objects ####
    #++ if _globals.debug: print " * Unregister references TO other objects..."
    for ref in self.getRefToObjs(REQUEST):
      ob = self.getLinkObj(ref)
      if ob is not None:
        ob.unregisterRefObj(self,REQUEST)

    ##### Unregister references FROM other objects ####
    #++ if _globals.debug: print " * Unregister references FROM other objects..."
    for ref in self.getRefByObjs(REQUEST):
      ob = self.getLinkObj(ref)
      if ob is not None:
        ob.unregisterRefToObj(self,REQUEST)

    # Process tree.
    for ob in self.getChildNodes(REQUEST):
      ob.onDeleteRefObjs(REQUEST,deep+1)


  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.onMoveRefObj:
  #
  #  This method is executed after an object has been moved.
  # ---------------------------------------------------------------------------------------------
  def onMoveRefObj(self, REQUEST, deep=0):
    #++ if _globals.debug: print "[%s.onMoveRefObj]: %s"%(self.meta_type,self.id)

    ##### Update references TO other objects ####
    for ref in self.getRefToObjs(REQUEST):
      ob = self.getLinkObj(ref)
      if ob is not None:
        ob.refreshRefObj(self,REQUEST)

    ##### Update references FROM other objects ####
    for ref in self.getRefByObjs(REQUEST):
      ob = self.getLinkObj(ref)
      if ob is not None:
        ob.refreshRefToObj(self,REQUEST)

    # Process tree.
    for ob in self.getChildNodes(REQUEST):
      ob.onMoveRefObj(REQUEST,deep+1)


  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.onCopyRefObj:
  #
  #  This method is executed after an object has been copied.
  # ---------------------------------------------------------------------------------------------
  def onCopyRefObj(self, REQUEST, deep=0):
    #++ if _globals.debug: print "[%s.onCopyRefObj]: %s"%(self.meta_type,self.id)

    ##### Register copy in references TO other objects ####
    for ref in self.getRefToObjs(REQUEST):
      ob = self.getLinkObj(ref)
      if ob is not None:
        ob.registerRefObj(self,REQUEST)

    ##### Clear references FROM other objects ####
    self.clearRegisteredRefObjs(REQUEST)

    # Process tree.
    for ob in self.getChildNodes(REQUEST):
      ob.onCopyRefObj(REQUEST,deep+1)


  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.onRollbackRefObj:
  #
  #  This method is executed before an object is rolled-back.
  # ---------------------------------------------------------------------------------------------
  def onRollbackRefObj(self, REQUEST):
    #++ if _globals.debug: print "[%s.onRollbackRefObj]: %s"%(self.meta_type,self.id)
  
    ##### Unregister references TO other objects from work-version ####
    req = {'lang':REQUEST['lang'],'preview':'preview'}
    for ref in self.getRefToObjs(req):
      ob = self.getLinkObj(ref)
      if ob is not None:
        ob.unregisterRefObj(self,req)
        
    ##### Register references TO other objects from live-version ####
    req = {'lang':REQUEST['lang'],'preview':''}
    for ref in self.getRefToObjs(req):
      ob = self.getLinkObj(ref)
      if ob is not None:
        ob.registerRefObj(self,req)


  """
  ###############################################################################################
  ###  
  ###  Resolve Links 
  ### 
  ###############################################################################################
  """

  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.getLinkObj:
  #
  #  Resolves internal/external links and returns Object.
  # ---------------------------------------------------------------------------------------------
  def getLinkObj(self, url, REQUEST={}): 
    ob = None
    if isInternalLink(url):
      docElmnt = self.getDocumentElement()
      path = url[2:-1]
      i = string.find(path,'@')
      if i > 0:
        clientId = path[:i]
        path = path[i+1:]
        thisHome = self.getHome()
        clientHome = None
        if thisHome.aq_parent.id == clientId:
          clientHome = thisHome.aq_parent
        elif hasattr(thisHome,clientId):
          clientHome = getattr(thisHome,clientId)
        if clientHome is not None:
          docElmnt = clientHome.objectValues(['ZMS'])[0]
      ob = docElmnt.findObjId(path,REQUEST)
    return ob

  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.getLinkUrl:
  #
  #  Resolves internal/external links and returns URL.
  # ---------------------------------------------------------------------------------------------
  def getLinkUrl(self, url, REQUEST={}): 
    if isInternalLink(url):
      ob = self.getLinkObj(url,REQUEST)
      url = None
      if ob is not None:
        if ob.meta_type == 'ZMSFile':
          file = ob.getFile(REQUEST)
          if file is not None:
            url = file.getHref(REQUEST)
        else:
          url = ob.getHref2IndexHtml(REQUEST)
    return url

  # ---------------------------------------------------------------------------------------------
  #  ZReferableItem.getLinkHtml:
  #
  #  Resolves internal/external links and returns Html.
  # ---------------------------------------------------------------------------------------------
  def getLinkHtml(self, url, html='<a href="%s">&raquo;</a>', REQUEST={}):
    s = ''
    ob = self.getLinkObj(url,REQUEST)
    if ob is not None:
      if ob.isActive(REQUEST) and \
         ob.isVisible(REQUEST):
        url = ob.getHref2IndexHtml(REQUEST)
        s = html%url
    return s

###################################################################################################
